@@ -14,6 +14,8 @@ or FITNESS FOR A PARTICULAR PURPOSE. See the license files for details.
 from tropofy.database.db_manager import DbManager
 from tropofy.widgets import GridWidget
 from datetime import date, time, datetime
+from sqlalchemy.sql.expression import desc, or_, cast
+from sqlalchemy import String
 
 PYTHON_TYPE_TO_FORM_TYPE = {  # SND: Remove this! Duplicated functionality in FormWidget
     int: 'text',
@@ -66,10 +68,49 @@ class SimpleGrid(GridWidget):
         edited_obj = DbManager.update_object_in_db(source_class=self.source_class, data=data)
         return edited_obj.as_json_data_row(self.get_grids_column_names())
 
-    def get_refresh_objects(self, data_set):
-        if hasattr(self.source_class, 'data_set_id') and data_set:
-            return data_set.query(self.source_class).all()
-        return []  # If an app could not filter by data_set_id - it would go here!
+    def get_filtered_refresh_objects(self, data_set, display_start, display_length, global_search_field, ordered_col_name_sort_direction_tuples):
+        if not data_set:
+            return [], 0, 0
+        qry = self.get_qry_of_non_filtered_refresh_objects(data_set)
+        total_records_before_filtering = qry.count()
+
+        #Filter from search
+        if (global_search_field):
+            search_conditions = []
+            do_not_search_on_cols = self.get_col_names_to_not_search_on(data_set)
+            for col_name in DbManager.sqla_cls_defined_column_names(self.source_class):
+                if col_name not in do_not_search_on_cols:
+                    search_conditions.append(
+                        cast(getattr(self.source_class, col_name), String).ilike('%' + global_search_field + '%')  # .ilike is case insensitive .like
+                    )
+            qry = qry.filter(or_(*search_conditions))
+                    
+
+        total_records_after_filtering = qry.count()
+        
+        #Order cols
+        for col_name, direction in ordered_col_name_sort_direction_tuples:
+            if direction == GridWidget.ASCENDING:
+                qry = qry.order_by(getattr(self.source_class, col_name))
+            elif direction == GridWidget.DESCENDING:
+                qry = qry.order_by(desc(getattr(self.source_class, col_name)))
+
+        #Reduce to num rows on grid page
+        qry = qry.offset(display_start).limit(display_length)
+
+        return qry.all(), total_records_before_filtering, total_records_after_filtering
+
+    def get_qry_of_non_filtered_refresh_objects(self, data_set):
+        """
+        :returns: SQLA qry object, that if executed, would return all objects in the grid - unfiltered and unsorted.
+        Assumes data_set exists.
+        """
+        if hasattr(self.source_class, 'data_set_id'):
+            return data_set.query(self.source_class)
+        raise AttributeError('SimpleGrid for source_class `%s` has no `data_set_id` column. SimpleGrid expects this. If not using data sets, extend SimpleGrid and overide `get_qry_of_non_filtered_refresh_objects`' % self.source_class.__name__)
+
+    def get_col_names_to_not_search_on(self, data_set):
+        return self.get_hidden_column_names()
 
     def get_type(self):
         return 'SimpleGrid'
